@@ -307,6 +307,7 @@ static void usershell(void) {
  * 									*
  ************************************************************************/
 static void usage(const char *name) {
+    /* split into 2 printf's to avoid overly long string (gcc warning) */
     printf("\
 Usage : %s [options] catalogfile entities...\n\
 \tParse the catalog file and query it for the entities\n\
@@ -314,13 +315,14 @@ Usage : %s [options] catalogfile entities...\n\
 \t--shell : run a shell allowing interactive queries\n\
 \t--create : create a new catalog\n\
 \t--add 'type' 'orig' 'replace' : add an XML entry\n\
-\t--add 'entry' : add an SGML entry\n\
+\t--add 'entry' : add an SGML entry\n", name);
+    printf("\
 \t--del 'values' : remove values\n\
 \t--noout: avoid dumping the result on stdout\n\
 \t         used with --add or --del, it saves the catalog changes\n\
 \t         and with --sgml it automatically updates the super catalog\n\
 \t--no-super-update: do not update the SGML super catalog\n\
-\t-v --verbose : provide debug informations\n", name);
+\t-v --verbose : provide debug informations\n");
 }
 int main(int argc, char **argv) {
     int i;
